from questionary import prompt
from guessit import guessit

from nyaacli.colors import red, green, prompt_style

from urllib import parse, request
from typing import Optional, List, Tuple, Dict
from dataclasses import dataclass
from datetime import datetime
from time import mktime
import feedparser
import logging
import sys
import os
import re

os.environ['REGEX_DISABLED'] = '1'

logger = logging.getLogger('nyaa')


def get_file_extension(path: str) -> str:
    """
    Gets the File extension from the path to a file

    'asd.txt' -> '.txt'
    'asd' -> ''
    '/path/to/asd.mkv' -> '.mkv'
    """
    filename, extension = os.path.splitext(path)

    return extension


def text_break() -> None:
    print('-' * 10, end='\n')


@dataclass
class Entry:
    title: str
    link: str
    original_title: str
    extension: str
    full_title: Optional[str]
    episode: Optional[int]
    other: Optional[str]
    release_group: Optional[str]
    screen_size: Optional[str]
    alternative_title: Optional[str]
    display_title: Optional[str]
    season: Optional[str]
    seeders: Optional[str]
    size: Optional[str]
    episode_title: Optional[str]
    date: datetime

    def __str__(self):
        return f'Entry(title={repr(self.title)}, episode={repr(self.episode)}, screen_size={repr(self.screen_size)})'


def search_torrent(
    search: str,
    episode: Optional[int] = None,
    dub: bool = False,
    number: int = 10,
    trusted_only: bool = False,
    sort_by: str = 'seeders'
) -> Optional[Tuple[str, str]]:
    """
    Results a tuple with (Path to .torrent file, Result name of video file)
    Nyaa.si rss search flags
    https://github.com/nyaadevs/nyaa/blob/a38e5d5b53805ecb1d94853d849826f948f07aad/nyaa/views/main.py#L65
    """

    search_query = f'{search}'.strip()
    if episode:
        search_query += f' {episode}'

    logger.debug(f'Searching nyaa for query: \'{search_query}\'')

    url_arguments: Dict[str, str] = {
        'c': '1_2',  # Language (English)
        'q': search_query,  # Search Query
    }

    if trusted_only:
        url_arguments['f'] = '2'  # Trusted uploaders only

    # What to sort torrents by
    if sort_by == 'seeders':
        url_arguments['s'] = 'seeders'
    elif sort_by == 'date':
        url_arguments['s'] = 'id'
    elif sort_by == 'size':
        url_arguments['s'] = 'size'
    elif sort_by == 'comments':
        url_arguments['s'] = 'comments'

    logger.debug(f'URL Arguments: {url_arguments}')

    search_url = 'https://nyaa.si/rss?' + parse.urlencode(url_arguments)

    # Parse Nyaa.si rss feed search
    feed: feedparser.FeedParserDict = feedparser.parse(search_url)
    logger.debug(f'Getting feed parse from: \'{search_url}\'')

    if not feed.entries:
        print(red('[Error] No entries found for search'))
        sys.exit(1)

    entries: List[Entry] = []

    # Find resolution in string regex
    screen_size_pattern = re.compile(r'^([0-9]+)|.?([0-9]{4})p')

    for entry in feed.entries:
        title = guessit(entry['title'])

        if episode:
            if title.get('type') != 'episode':
                logger.debug(f"Skipping (Not episode, but searching for E{episode}) {entry}")
                continue

            if title.get('episode') != episode:
                logger.debug(f"Skipping (Wrong episode, searching for E{episode}) {entry}")
                continue

        if not dub and 'dub' in entry['title'].lower():
            # Ignore entries with 'dub' in their titles if dub=False
            logger.debug(f"Skipping (Dub) {entry}")
            continue

        size = 0

        # Screen size needs to be higher than 480p
        if title.get('screen_size'):
            sizes = re.search(screen_size_pattern, title.get('screen_size'))
        else:
            sizes = re.search(screen_size_pattern, entry['title'])

        if sizes:
            for group in sizes.groups():
                if group:
                    size = int(group)

        good_size = size > 480

        if not title.get('screen_size'):
            # Add screen size to title properties if not already there
            title['screen_size'] = f'{size}p'

        if not good_size:
            logger.debug(f"Skipping (bad size ({size})) {entry}")
            continue

        entry = Entry(
            link=entry['link'],
            size=entry['nyaa_size'],
            original_title=entry['title'],
            display_title=entry['title'],
            extension=get_file_extension(entry['title']),
            title=title.get('title'),
            season=title.get('season'),
            seeders=entry.get('nyaa_seeders'),
            full_title='',
            episode=title.get('episode'),
            episode_title=title.get('episode_title'),
            other=title.get('other'),
            release_group=title.get('release_group'),
            screen_size=title.get('screen_size'),
            alternative_title=title.get('alternative_title'),
            date=datetime.fromtimestamp(mktime(entry.get('published_parsed')))
        )
        logger.debug(f'Added entry: {entry}')
        entries.append(entry)

    if not entries:
        print(red(f'No results found for search: \'{search_query.replace("%20", " ")}\''))
        return None

    for entry in entries[:number]:
        entry_title = entry.title
        entry.full_title = entry.title

        if entry.alternative_title:
            entry_title += f' - {entry.alternative_title}'
            entry.full_title += f' - {entry.alternative_title}'

        if entry.episode or entry.episode_title:
            if entry.episode:
                entry_title += f' - Episode {entry.episode}'
                entry.full_title += f' - Episode {entry.episode}'
            else:
                entry_title += f' - Episode {entry.episode_title}'
                entry.full_title += f' - Episode {entry.episode_title}'

        if entry.season:
            entry_title += f' - Season {entry.season}'
            entry.full_title += f' - Season {entry.season}'

        if entry.release_group:
            entry_title += f' ({entry.release_group})'

        if entry.screen_size:
            entry_title += f' {entry.screen_size}'

        if entry.seeders:
            seeders = f'{entry.seeders} Seeders'
            entry_title += f' - {seeders}'

        if entry.size:
            entry_title += f' - {entry.size}'

        if entry.date:
            entry_title += f" ({entry.date.strftime('%d/%m/%y')})"

        entry.display_title = entry_title

    choices = [{'name': entry.display_title, 'value': index} for index, entry in enumerate(entries[:number])]

    questions = [
        {
            'type': 'list',
            'choices': choices,
            'name': 'selection',
            'message': 'Select one of the entries below',
        }
    ]

    logger.debug(f'Download choices: {choices}')

    answer = prompt(questions, style=prompt_style)

    if not answer:
        # Cancelled with Ctrl + C
        sys.exit(0)

    logger.debug(f'Selection: [{answer["selection"]}]')

    index_choice = answer['selection']

    entry_choice = entries[index_choice]

    logger.debug(f'Selected entry at index {index_choice}: {entry_choice}')

    final_path = entry_choice.full_title

    torrent_path = f'/tmp/{final_path}.torrent'
    logger.debug(f'Downloading torrent file to \'{torrent_path}\' from \'{entry_choice.link}\'')

    print(f"{green('[Downloading Torrent File]')} '{torrent_path}'")

    request.urlretrieve(entry_choice.link, torrent_path)
    logger.debug('Downloaded torrent file')

    return torrent_path, final_path + entry_choice.extension



#!/usr/bin/env python

import logging
import sys
from typing import List

from nyaacli.nyaa_search import search_torrent
from nyaacli.colors import red, green
from nyaacli.logger import CustomFormatter
from nyaacli.utils import xdg_open

import click
from colorama import init

init()


def entries_autocomplete(ctx, args: List[str], incomplete: str):
    """
    Auto-complete choices for --number / -n argument
    """

    text = 'Selection Entries'

    entries = [
        ('5', f'5 {text}'),
        ('10', f'10 {text} (default)'),
        ('15', f'15 {text}'),
        ('20', f'20 {text}')
    ]

    return [c for c in entries if incomplete in c[0]]


def sort_mode_autocomplete(ctx, args: List[str], incomplete: str):
    """
    Auto-complete choices for --sort-by / -s argument
    """

    entries = [
        ('seeders', 'Sort by number of torrent seeders (default)'),
        ('date', 'Sort by upload date'),
        ('size', 'Sort by file size'),
        ('comments', 'Sort by number of comments')
    ]

    return [c for c in entries if incomplete in c[0]]


@click.command()
@click.argument('anime')
@click.argument('episode', type=int, default=None, required=False)
@click.option('--output', '-o', default='~/Videos/Anime', help=green('Output Folder'), type=click.Path(), show_default=True)
@click.option('--number', '-n', default=10, help=green('Number of entries'), show_default=True, autocompletion=entries_autocomplete)
@click.option('--sort-by', '-s', default='seeders', help=green('Sort by'), show_default=True, autocompletion=sort_mode_autocomplete)
@click.option('--trusted', '-t', default=False, help=green('Only search trusted uploads'), is_flag=True)
@click.option('--debug', '-d', default=False, help=green('Debug Mode'), is_flag=True)
@click.option('--client', '-c', default=False, help=green('Use Torrent Client'), is_flag=True)
def main(
    anime: str,
    episode: int,
    output: str,
    debug: bool = False,
    client: bool = False,
    trusted: bool = False,
    number: int = 10,
    sort_by: str = 'seeders',
):
    """
    Search for Anime on https://nyaa.si and downloads it

    \b
    Usage:
        \33[92mnyaa \33[36m"Anime Name" \33[33m<Episode Number (Optional)> \33[34m-o <Output Folder (Default = "~/Videos/Anime")>\033[0m

    \b
    Example:
        \33[92mnyaa \33[36m"Kimetsu no Yaiba" \33[33m19 \33[34m-o /home/user/My/Animes/Folder/Kimetsu_No_Yaiba/\033[0m
    """
    # Setup debugging
    logger = logging.getLogger("nyaa")
    logger.setLevel(logging.CRITICAL)

    ch = logging.StreamHandler()
    ch.setLevel(logging.CRITICAL)
    ch.setFormatter(CustomFormatter())

    logger.addHandler(ch)

    if debug:
        # Add debugging logging in debug mode
        logger.setLevel(logging.DEBUG)
        ch.setLevel(logging.DEBUG)

    torrent_search = search_torrent(anime, episode, number=number, trusted_only=trusted, sort_by=sort_by)

    if torrent_search:
        torrent_path, result_name = torrent_search

        if client:
            xdg_open(torrent_path)
        else:
            download_torrent(torrent_path, result_name, base_path=output)


try:
    from nyaacli.torrenting import download_torrent
except ModuleNotFoundError:
    print(red("You need to have the 'libtorrent' library (with the Python API) installed to user nyaa-cli.\n"))

    print('- Install with apt:', green('sudo apt install python3-libtorrent'))
    print('- Install with pacman:', green('sudo pacman -S libtorrent-rasterbar'))

    libtorrent_url = green("https://github.com/arvidn/libtorrent/blob/RC_1_2/docs/python_binding.rst")

    print(f"\nOtherwise, look into how you can build it from source here: {libtorrent_url}")
    sys.exit(1)

main()



from colorama import Fore

import logging


class CustomFormatter(logging.Formatter):
    """
    https://stackoverflow.com/a/56944256/10416161
    Logging Formatter to add colors and count warning / errors
    """

    log_format = "%(levelname)s - %(message)s (%(filename)s:%(lineno)d)"

    FORMATS = {
        logging.DEBUG: Fore.LIGHTWHITE_EX + log_format + Fore.RESET,
        logging.INFO: Fore.LIGHTWHITE_EX + log_format + Fore.RESET,
        logging.WARNING: Fore.YELLOW + log_format + Fore.RESET,
        logging.ERROR: Fore.RED + log_format + Fore.RESET,
        logging.CRITICAL: Fore.RED + log_format + Fore.RESET
    }

    def format(self, record: logging.LogRecord):
        log_fmt = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_fmt)

        return formatter.format(record)

